use ron::value::Value;
use serde::{Serialize, Deserialize};

use std::env::args;

use rustible::Facts;
use rustible::modules::{
    Module, PlaybookRun, ModuleSetup,
    package::{PackageModule as Package, PackageState, PackageError, PackageArgs},
    command::{CommandModule as Command, CommandError, CommandArgs},
};


#[derive(Clone, Debug, PartialEq, Eq, Serialize, Deserialize)]
pub struct DeclarativePlaybook {
    modules: Vec<DeclarativeModule>,
}

#[derive(Clone, Debug, PartialEq, Eq, Serialize, Deserialize)]
pub struct DeclarativeModule {
    name: String,
    module: String,
    args: Value,
}

#[derive(Clone, Debug, serde::Serialize)]
pub enum PlaybookError {
    Command(CommandError),
    Package(PackageError),
}

impl From<CommandError> for PlaybookError {
    fn from(e: CommandError) -> PlaybookError {
        PlaybookError::Command(e)
    }
}

impl From<PackageError> for PlaybookError {
    fn from(e: PackageError) -> PlaybookError {
        PlaybookError::Package(e)
    }
}

fn main() -> Result<(), PlaybookError> {
    let facts = Facts::new();
    
    let mut playbook = PlaybookRun::new();

    let mut cli_args = args();
    let test_playbook = cli_args.nth(1).expect("First argument should be playbook.ron");
    let test_shell = cli_args.nth(0).expect("Second argument should be test.sh");

    let test_playbook_content = std::fs::read_to_string(&test_playbook).unwrap();

    let playbook_ron: DeclarativePlaybook = ron::from_str(&test_playbook_content).unwrap();

    for task in playbook_ron.modules {
        println!("{}", task.name);
        match task.module.as_str() {
            "command" => {
                let args: CommandArgs = task.args.into_rust().unwrap();
                let module = args.with_facts(&facts);
                playbook.run(module)?;
                //println!("{:?}", module);
            }, "package" => {
                let args: PackageArgs = task.args.into_rust().unwrap();
                let module = args.with_facts(&facts);
                playbook.run(module)?;
                //println!("{:?}", module);
            }, _ => {
                unimplemented!();
            }
        }
    }

    Ok(())
}
